/*
   script.c -- PIKT run script/alarm routines

   $Header: /var/cvs/PIKT/pikt/src/piktd/script.c,v 1.20 2004/01/29 15:06:14 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998,1999,2000,2001,2002,2003,2004 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
*/

#include "../lib/common.h"

#include "piktd.h"

char inlin[2][MAXLINE+1];
char *inlinnow, *inlinprc;
int innum;
int skip;
int eoi;
char *outlinnow, *outlinprc;

/* do some initializations; for each statement in the script begin section,
   if any, process the statement, and proceed through the rest of the begin
   section as indicated
 */
int dobegin (void)
{
	static char fnnam[] = "dobegin";

	eoi = FALSE;
	inlen = 0;
	inlinnow = inlinprc = (char *) NULL;
	inlin[0][0] = inlin[1][0] = '\0';
	outlinnow = outlinprc = (char *) NULL;
	innum = 0;
	FREEVOID(keystr);
	keystrlen = 0;
	if (BEGINSECTION) {
		if (isdebug)
			logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "BEGIN");
		rulcurr = BEGINSECTION;
		stmcurr = BEGINSECTION->statements;
		while (stmcurr) {
			switch (dostatement(stmcurr->type)) {
				case TRUE:
					stmcurr = stmcurr->tnext;
					break;
				case FALSE:
					stmcurr = stmcurr->fnext;
					break;
				case AGAIN:
					stmcurr = BEGINSECTION->statements;
					break;
				case QUIT:
					return QUIT;
					break;
				default:
					logreturn(PIKTLOG, LOG_ERR, ERRID_STATEMENT_RETURN_INVALID, __FILE__, __LINE__, fnnam, FALSE, "in %s, invalid dostatement() return at begin", ALMNAM);
					STATUS = INACTIVE;
					return ERROR;
					break;
			}
		}
	}
	return 0;
}

/* do some initializations; for each statement in the script end section,
   if any, process the statement, and proceed through the rest of the end
   section as indicated
 */
int
doend (void)
{
	static char fnnam[] = "doend";

	eoi = TRUE;
	inlen = 0;
	inlinnow = (char *) NULL;
	outlinnow = (char *) NULL;
	FREEVOID(keystr);
	keystrlen = 0;
	if (ENDSECTION) {
		if (isdebug)
			logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "END");
		rulcurr = ENDSECTION;
		stmcurr = ENDSECTION->statements;
		while (stmcurr) {
			switch (dostatement(stmcurr->type)) {
				case TRUE:
					stmcurr = stmcurr->tnext;
					break;
				case FALSE:
					stmcurr = stmcurr->fnext;
					break;
				case AGAIN:
					stmcurr = ENDSECTION->statements;
					break;
				case QUIT:
					return QUIT;
					break;
				default:
					logreturn(PIKTLOG, LOG_ERR, ERRID_STATEMENT_RETURN_INVALID, __FILE__, __LINE__, fnnam, FALSE, "in %s, invalid dostatement() return at end", ALMNAM);
					STATUS = INACTIVE;
					return ERROR;
					break;
			}
		}
	}
	return 0;
}

/* do some initializations; if there is script input, also script rules
   sections, for each line of input: skip ahead, if indicated; scan the input,
   assigning dat values; make history lookup key for current input; for each
   rule, and for each statement in a rule, process the statement, record
   history values for current statement, and proceed through the rest of
   the rules sections as indicated
 */
int
dorules (int inputtype, FILE *infile, int postype)
{
	static char fnnam[] = "dorules";
	signed char c;
	char *oln;

	skip = 0;
	inlinnow = inlin[0];
	if ((inputtype != NOINPUT) && RULESSECTION) {
		while (fgets(inlinnow, MAXLINE+1, infile) != NULL) {
			outlinnow = inlinnow;
			innum++;
			if (skip) {
				skip--;
				continue;
			}
			/* determine if at end of input */
			if ((c = fgetc(infile)) == EOF)
				eoi = TRUE;
			else
				ungetc(c, infile);
			if (scaninput(postype) == ERROR)
				continue;
			keystrlen = 0;
			if (makekeystr() == ERROR)
				goto _QUIT;
			_REDO:
			rulcurr = RULESSECTION;
			while (rulcurr) {
				_AGAIN:
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "RULE");
				stmcurr = rulcurr->statements;
				while (stmcurr) {
					switch (dostatement(stmcurr->type)) {
						case TRUE:
							stmcurr = stmcurr->tnext;
							break;
						case FALSE:
							stmcurr = stmcurr->fnext;
							break;
						case AGAIN:	/* redo current rule */
							goto _AGAIN;
							break;
						case LEAVE:	/* leave current rule */
							goto _LEAVE;
							break;
						case REDO:	/* redo current line */
							goto _REDO;
							break;
						case NEXT:	/* go on to next line */
							goto _NEXT;
							break;
						case SKIP:	/* go ahead n lines */
							goto _SKIP;
							break;
						case LAST:	/* break to end */
							if (advanceinput() == ERROR)
								goto _QUIT;
							goto _LAST;
							break;
						case QUIT:	/* go on to next alarm */
							return QUIT;
							break;
						default:
							logreturn(PIKTLOG, LOG_ERR, ERRID_STATEMENT_RETURN_INVALID, __FILE__, __LINE__, fnnam, FALSE, "in %s, invalid dostatement() return in rule", ALMNAM);
							STATUS = INACTIVE;
							return ERROR;
							break;
					}
				}
				_LEAVE:
				rulcurr = rulcurr->next;
			}
			_NEXT:
			_SKIP:
			if (INPUTTMPFIL && outlinnow && strcmp(outlinnow,nilstr)) {
				oln = mystrdup(outlinnow, fnnam);
				requote(oln);
				if (fprintf(intmpfp, "%s\n", oln) < 0) {
					free(oln);
					REWRITEINFIL = FALSE;
					logreturn(PIKTLOG, LOG_ERR, ERRID_FILE_WRITE_ERROR, __FILE__, __LINE__, fnnam, FALSE, "in %s, fprintf() error writing to %s for: %s", ALMNAM, INPUTTMPFIL, outlinnow);
					STATUS = INACTIVE;
					return ERROR;
				}
				free(oln);
			}
			if (recordvalnows() == ERROR)
				goto _QUIT;
			if (advanceinput() == ERROR)
				goto _QUIT;
		} /* while (fgets(inlinnow, MAXLINE+1, infile) != NULL) */
	}
	_LAST:
	if (infile && (innum == 0) && RULESSECTION)
		logreturn(PIKTLOG, LOG_WARNING, ERRID_NO_INPUT_SOURCE, __FILE__, __LINE__, fnnam, FALSE, "in %s, no input data", ALMNAM);
	_QUIT:
	return 0;
}

/* advance the input line; set valprc to preceding line's valnow; garbage
   collect preceding line's valnows; carry forward preceding line's free vals
 */
int
advanceinput (void)
{
	static char fnnam[] = "advanceinput";
	val *valtmp;
	char *inlintmp;
	register int i, j, k;
	vl *vlp, *nvlp, *initvlp;

	/* switch inlinnow/inlinprc */
	if (inlinprc) {
		inlintmp = inlinprc;
		inlinprc = inlinnow;
		inlinnow = inlintmp;
	} else {
		inlinprc = inlin[0];
		inlinnow = inlin[1];
	}
	/* switch outlinnow/outlinprc */
	outlinprc = outlinnow;
	outlinnow = (char *) NULL;
	/* switch valnow/valprc */
	valtmp = valprc;
	valprc = valnow;
	valnow = valtmp;
	/* garbage collect preceding line's valnows */
	freevals(&valnow, FIXEDVAL|FREEVAL);
	/* carry forward free vals from preceding (last) input line */
	for (i=1; i<=VRNDXHI; i++) {
		if (VALTYP(i) & FREEVAL) {
			for (j=0; j<=9; j++) {
				vlp = initvlp = valprc->vals[i][j];
				while (vlp) {
					if (! (nvlp = regvl(valnow, i, vlp->ndx, vlp->key, ARYTYP(i)))) {
						logreturn(PIKTLOG, LOG_ERR, ERRID_REGVL_ERROR, __FILE__, __LINE__, fnnam, FALSE, "in %s, regvl() error for var %s", ALMNAM, VARNAM(i));
						return ERROR;
					}
					for (k=0; k<=MAXARYDIM; k++)
						nvlp->ndx[k] = vlp->ndx[k];
					if (vlp->key)
						nvlp->key = mystrdup(vlp->key, fnnam);
					nvlp->num = vlp->num;
					if (vlp->str)
						nvlp->str = mystrdup(vlp->str, fnnam);
					nvlp->fil = vlp->fil;
					vlp = vlp->next;
					if (vlp == initvlp)
						break;
				}
			}
		}
	}
	return 0;
}
